pub struct Files { /* private fields */ }Expand description
Endpoint for serving files based on incoming GET requests.
Implementations
Sourceimpl Files
impl Files
Sourcepub fn new(root: &str) -> Files
pub fn new(root: &str) -> Files
Create new endpoint, serving files from directory pointed by root.
Sourcepub fn require_permission(&mut self, permission: &String)
pub fn require_permission(&mut self, permission: &String)
Require permission label required in incoming JWT token.
If incoming request does not have JWT token, or the token does not include this permission label in its claims, the request is rejected as unauthorized.